@@ -53,6 +53,7 @@ from .ops.enum import *
 from .ops.norm import *
 from .ops.quant import *
 from .ops.gemm_op_a8w8 import *
+from .ops.gemm_op_a16w16 import *
 from .ops.gemm_op_a4w4 import *
 from .ops.batched_gemm_op_a8w8 import *
 from .ops.batched_gemm_op_bf16 import *
@@ -61,6 +62,7 @@ from .ops.activation import *
 from .ops.attention import *
 from .ops.custom import *
 from .ops.custom_all_reduce import *
+from .ops.quick_all_reduce import *
 from .ops.moe_op import *
 from .ops.moe_sorting import *
 from .ops.pos_encoding import *
@@ -72,4 +74,5 @@ from .ops.topk import *
 from .ops.mha import *
 from .ops.gradlib import *
 from .ops.trans_ragged_layout import *
+from .ops.sample import *
 from . import mla
