@@ -15,6 +15,7 @@
 * limitations under the License.
 """
 
+import functools
 import os
 import random
 from pathlib import Path
@@ -24,9 +25,11 @@ import pandas as pd
 from aiter import dtypes
 import torch
 import torch.nn.functional as F
-from aiter.test_common import perftest
+
 from aiter.utility.mp_tuner import mp_tuner
 from functools import lru_cache
+from aiter.jit.core import get_asm_dir
+from aiter.jit.utils.chip_info import get_cu_num
 
 aiter.rocb_create_extension()
 aiter.hipb_create_extension()
@@ -60,6 +63,25 @@ def call_rocb_mm(inp, w, solidx):
     return aiter.rocb_mm(inp, w, solidx)
 
 
+def run_gemm_bf16_asm(inp, w, out, bias=None, splitK=None, kernelName=None):
+    return aiter.gemm_a16w16_asm(
+        inp, w, out, bias=bias, splitK=splitK, kernelName=kernelName
+    )
+
+
+@functools.lru_cache(maxsize=1024)
+def compute_gemm_SplitK(M: int, N: int, K: int, tile_m: int, tile_n: int, tile_k: int):
+    cu_num = get_cu_num()
+    tile_num = ((M + tile_m - 1) // tile_m) * ((N + tile_n - 1) // tile_n)
+    cusPerTile = cu_num / tile_num
+    splitK = 0
+    if tile_num < cu_num:
+        splitK = int(cu_num / tile_num)
+    else:
+        splitK = 4
+    return splitK
+
+
 def generate_data(
     m, n, k, indtype, outdtype, scaleAB, seed=0, bias=None, device="cuda:0"
 ):
@@ -71,7 +93,8 @@ def generate_data(
     scale_half = torch.tensor(0.5, dtype=dtypes.fp32, device=device)
     scale_one = torch.tensor(1, dtype=dtypes.fp32, device=device)
     scale = scale_half if scaleAB else scale_one
-    return (inp, weights, weights.t(), bias, scale)
+    out_asm = torch.empty(m, n, dtype=outdtype, device=device)
+    return (inp, weights, weights.t(), bias, scale, out_asm)
 
 
 def get_gemm_ref(inp, weights, bias, scale, indtype, outdtype):
@@ -121,6 +144,8 @@ class Gemm:
         rocblas_decode=False,
         mp=1,
         err_ratio=0.01,
+        profile_file="",
+        # splitK=None,
     ):
         self.m = m
         self.k = k
@@ -131,7 +156,7 @@ class Gemm:
         self.scaleAB = scaleAB
         self.use_rocblas = indtype == outdtype and str(indtype) != "dtypes.fp8"
         self.nb = CACHE_INVALIDATE_BUFFERS
-        (self.inp, self.weights, _, self.bias, scaleA) = generate_data(
+        (self.inp, self.weights, _, self.bias, _, scaleA) = generate_data(
             m, n, k, indtype, outdtype, scaleAB, 0, bias
         )
         self.blob = torch.ones(128 * 1024 * 1024, dtype=dtypes.fp32, device="cuda")
@@ -142,6 +167,8 @@ class Gemm:
         self.atol = 1e-2
         self.ref = self.get_gemm_ref()
         self.check_err_ratio = err_ratio
+        self.splitK = None
+        self.profile_file = profile_file
         self.start = torch.cuda.Event(enable_timing=True)
         self.end = torch.cuda.Event(enable_timing=True)
         # prefer hipblaslt unless rocblas time is less than this
@@ -151,6 +178,7 @@ class Gemm:
         self.mp = mp
         self.inbpe = self.inp.element_size()
         self.outbpe = self.ref.element_size()
+        self.asm_map = {}
 
     def calculate_perf(
         self,
@@ -161,10 +189,10 @@ class Gemm:
         """calculate TFLOPS and bandwidth"""
         ### gemm flops,bw
         info, time, err_ratio = results
-        if time == -1:
+        if time <= 0:
             return -1, -1
         print("info is ", info)
-        cu_num, m, n, k = info[0]
+        cu_num, m, n, k = info
         flops = m * n * k * 2
         tflops = round(flops / (time * 1000000), 2)
 
@@ -229,6 +257,118 @@ class Gemm:
             ref = F.linear(self.inp, self.weights, self.bias).to(self.outdtype)
         return ref
 
+    def get_asm_kernels(self, file):
+        if not os.path.exists(file):
+            print(f"ASM kernel list file not exist: {file}")
+            return {}
+        df = pd.read_csv(file)
+        kernel_dict = (
+            df.groupby(["tileM", "tileN", "pf"])["knl_name"].apply(list).to_dict()
+        )
+        return kernel_dict
+
+    def asm_gemm_all_solutions(self):
+        if (
+            self.scaleAB
+            # or self.k % 64 != 0
+            or (not (self.indtype == dtypes.bf16 and self.outdtype == dtypes.fp32))
+        ):
+            self.asm_gtimedf = pd.DataFrame(columns=["gtimems", "libtype"])
+            return
+        asm_kernel_list_csv = f"{get_asm_dir()}/bf16gemm/bf16gemm_outf32.csv"
+        asm_kernels = self.get_asm_kernels(asm_kernel_list_csv)
+        asm_tiles = [key for key in asm_kernels.keys()]
+        solidx = 0
+        task_asm = []
+
+        solutions = 0
+        for key in asm_tiles:
+            tile_m, tile_n, pf = key
+            print(f"ASM Tile - M: {tile_m}, N: {tile_n}, PF: {pf}")
+            kernelName = asm_kernels[key][0]
+            maxSplitK = compute_gemm_SplitK(
+                self.m, self.n, self.k, tile_m, tile_n, 256
+            )  # if self.splitK else 1
+            solidx = solidx + 1
+            self.asm_map[solidx] = kernelName
+            for splitK in range(1, maxSplitK + 1):
+                info = (
+                    (
+                        self.m,
+                        self.n,
+                        self.k,
+                        False,
+                        self.indtype,
+                        self.outdtype,
+                        self.scaleAB,
+                    ),
+                    solidx,
+                    splitK,
+                    "asm",
+                    kernelName,
+                )
+                task_asm.append(
+                    (
+                        info,
+                        generate_data,
+                        (
+                            self.m,
+                            self.n,
+                            self.k,
+                            self.indtype,
+                            self.outdtype,
+                            self.scaleAB,
+                        ),
+                        run_gemm_bf16_asm,
+                        ([0, 1, 5, 3], splitK, kernelName),
+                        {},
+                        get_gemm_ref,
+                        ([0, 1, 3, 4], self.indtype, self.outdtype),
+                        {},
+                        None,  # self.ref if fast_mode == 0 else None,
+                        self.rtol,
+                        self.atol,
+                    )
+                )
+
+                solutions = solutions + 1
+        in_data = [
+            (
+                solutions,
+                (),
+            )
+        ]
+        gtimes = {}
+        ret = mp_tuner(task_asm, in_data, self.mp, False)
+
+        results = []
+        for info, us, err_ratio in ret:
+            solidx = info[1]
+            splitK = info[2]
+            kernelName = info[4]
+            res_one = []
+            # if err_ratio > self.check_err_ratio:
+            #    continue
+            gtimes["solidx"] = solidx
+            gtimes["gtimems"] = us / 1000.0
+            gtimes["splitK"] = splitK
+            res_one.append(solidx)
+            res_one.append(round(us / 1000.0, 4))
+            res_one.append(splitK)
+            res_one.append(err_ratio)
+            res_one.append(kernelName)
+            results.append(res_one)
+
+        self.asm_gtimedf = pd.DataFrame(
+            results, columns=["solidx", "gtimems", "splitK", "err_ratio", "kernelName"]
+        )
+        self.asm_gtimedf["libtype"] = "asm"
+        print(self.asm_gtimedf)
+        self.asm_gtimedf.to_csv("/tmp/asm_gtimedf.csv", index=False)
+        self.asm_gtimedf = self.asm_gtimedf.sort_values(by="gtimems")
+        print(">>> asm top solutions, Fast Mode", 0)
+        print(self.asm_gtimedf.head(self.topn))
+
     def hipb_time_all_sols(self, fast_mode=0, top_sols=0):
         coldi = 20
         warmi = 20
@@ -255,6 +395,9 @@ class Gemm:
                     self.scaleAB,
                 ),
                 solidx,
+                0,  # splitK
+                "hipblaslt",
+                "",
             )
             task.append(
                 (
@@ -282,19 +425,31 @@ class Gemm:
             )
         ]
         ret = mp_tuner(task, in_data, self.mp, fast_mode == 1)
+        results = []
         for info, us, err_ratio in ret:
+            res_one = []
             solidx = info[1]
+            kernelName = info[4]
             if fast_mode == 0:
                 if err_ratio > self.check_err_ratio:
                     continue
-            gtimes[solidx] = us / 1000.0
+            res_one.append(solidx)
+            res_one.append(round(us / 1000.0, 4))
+            res_one.append(err_ratio)
+            res_one.append(kernelName)
 
-        self.hipb_gtimedf = pd.DataFrame.from_dict(
-            gtimes, orient="index", columns=["gtimems"]
-        ).sort_values(by="gtimems")
-        self.hipb_gtimedf.to_csv("/tmp/hipb_gtimedf.csv")
+            results.append(res_one)
+        self.hipb_gtimedf = pd.DataFrame(
+            results, columns=["solidx", "gtimems", "err_ratio", "kernelName"]
+        )
+
+        self.hipb_gtimedf = self.hipb_gtimedf.sort_values(by="gtimems")
+        self.hipb_gtimedf["libtype"] = "hipblaslt"
+
+        self.hipb_gtimedf.to_csv("/tmp/hipb_gtimedf.csv", index=False)
         print(">>> HipBlasLt top solutions, Fast Mode", fast_mode)
         print(self.hipb_gtimedf.head(self.topn))
+        self.hipb_gtimedf["splitK"] = 0
 
     def find_rocblas_sols(self):
         if self.scaleAB or self.bias is not None:
@@ -330,6 +485,9 @@ class Gemm:
             info = (
                 (self.m, self.n, self.k, False, self.indtype, self.outdtype, False),
                 solidx,
+                0,
+                "rocblas",
+                "rocblas",
             )
             task.append(
                 (
@@ -355,16 +513,26 @@ class Gemm:
             )
         in_data = [(len(solutions), ())]
         ret = mp_tuner(task, in_data, self.mp, fast_mode == 1)
+        results = []
         for info, us, err_ratio in ret:
             solidx = info[1]
+            kernelName = info[4]
+            ret_one = []
             if fast_mode == 0:
                 if err_ratio > self.check_err_ratio:
                     continue
-            gtimes[solidx] = us / 1000.0
-        self.rocb_gtimedf = pd.DataFrame.from_dict(
-            gtimes, orient="index", columns=["gtimems"]
-        ).sort_values(by="gtimems")
-        self.rocb_gtimedf.to_csv("/tmp/rocb_gtimedf.csv")
+            ret_one.append(solidx)
+            ret_one.append(round(us / 1000.0, 4))
+            ret_one.append(err_ratio)
+            ret_one.append(kernelName)
+            results.append(ret_one)
+        self.rocb_gtimedf = pd.DataFrame(
+            results, columns=["solidx", "gtimems", "err_ratio", "kernelName"]
+        )
+        self.rocb_gtimedf = self.rocb_gtimedf.sort_values(by="gtimems")
+        self.rocb_gtimedf["libtype"] = "rocblas"
+        self.rocb_gtimedf["splitK"] = 0
+        self.rocb_gtimedf.to_csv("/tmp/rocb_gtimedf.csv", index=False)
         print(">>> Rocblas top solutions, Fast Mode", fast_mode, flush=True)
         print(self.rocb_gtimedf.head(self.topn), flush=True)
 
@@ -373,13 +541,9 @@ class Gemm:
             self.blob = self.blob + 0.00001
 
     def functional_get_topn_fastest(self):
-        rocb_topn = []
-        for solidx in self.rocb_gtimedf.index[: self.topn]:
-            rocb_topn.append(solidx)
+        rocb_topn = self.rocb_gtimedf["solidx"].head(self.topn).tolist()
         self.rocb_top_sols = rocb_topn
-        hipb_topn = []
-        for solidx in self.hipb_gtimedf.index[: self.topn]:
-            hipb_topn.append(solidx)
+        hipb_topn = self.hipb_gtimedf["solidx"].head(self.topn).tolist()
         self.hipb_top_sols = hipb_topn
 
     def find_fastest_solution(self):
@@ -396,40 +560,139 @@ class Gemm:
         self.rocb_time_all_sols(fast_mode=0, top_sols=1)
         self.warmup()
         self.hipb_time_all_sols(fast_mode=0, top_sols=1)
+        self.asm_gemm_all_solutions()
+        if self.profile_file != "":
+            if os.path.exists(self.profile_file):
+                old_df = pd.read_csv(self.profile_file)
+            else:
+                old_df = pd.DataFrame(
+                    columns=[
+                        "M",
+                        "N",
+                        "K",
+                        "bias",
+                        "dtype",
+                        "outdtype",
+                        "scaleAB",
+                        "cu_num",
+                        "libtype",
+                        "solidx",
+                        "splitK",
+                        "soltimes",
+                        "kernelName",
+                        "err_ratio",
+                    ]
+                )
+
+            resultsdf = pd.concat(
+                [self.rocb_gtimedf, self.hipb_gtimedf, self.asm_gtimedf],
+                ignore_index=True,
+            )
+            resultsdf = resultsdf.rename(
+                columns={
+                    "gtimems": "soltimes",
+                }
+            )
+            resultsdf["soltimes"] = resultsdf["soltimes"].apply(
+                lambda x: round(x * 1000, 3)
+            )
+            print(resultsdf)
+            resultsdf["M"] = self.m
+            resultsdf["N"] = self.n
+            resultsdf["K"] = self.k
+            resultsdf["bias"] = self.bias
+            resultsdf["dtype"] = self.indtype
+            resultsdf["outdtype"] = self.outdtype
+            resultsdf["scaleAB"] = self.scaleAB
+            resultsdf["cu_num"] = get_cu_num()
+            keys = [
+                "cu_num",
+                "M",
+                "N",
+                "K",
+            ]
+            results = resultsdf.apply(
+                lambda row: self.calculate_perf(
+                    (
+                        tuple(row[col] for col in keys),
+                        row["soltimes"],
+                        row["err_ratio"],
+                    ),
+                    self.inbpe,
+                    self.outbpe,
+                ),
+                axis=1,
+                result_type="expand",
+            )
+            resultsdf["tflops"] = results[0]
+            resultsdf["bw"] = results[1]
+
+            resultsdf = pd.concat([old_df, resultsdf], ignore_index=True)
+            resultsdf.to_csv(self.profile_file, index=False)
+        if len(self.asm_gtimedf) > 0:
+            self.asm_gtimedf = self.asm_gtimedf[
+                self.asm_gtimedf["err_ratio"] < self.check_err_ratio
+            ]
+            print(self.asm_gtimedf)
+        if len(self.hipb_gtimedf) > 0 or len(self.asm_gtimedf) > 0:
+            self.hipb_gtimedf = pd.concat(
+                [self.hipb_gtimedf, self.asm_gtimedf], ignore_index=True
+            )
+        # get best solution
+        self.hipb_gtimedf = self.hipb_gtimedf.sort_values(by="gtimems")
+        print("rocb_gtimedf", self.rocb_gtimedf)
         if len(self.rocb_gtimedf) > 0 and len(self.hipb_gtimedf) > 0:
             best_rocb_time = self.rocb_gtimedf.gtimems.iloc[0]
             best_hipb_time = self.hipb_gtimedf.gtimems.iloc[0]
+            best_asm_time = 100  # self.asm_gtimedf.gtimems.iloc[0]
+            self.best_kernelName = ""
             if best_rocb_time < best_hipb_time * self.hipb_prefer_ratio:
-                self.best_libtype = "rocblas"
-                self.best_solidx = self.rocb_gtimedf.index[0]
+                self.best_libtype = self.rocb_gtimedf.libtype.iloc[0]
+                self.best_solidx = self.rocb_gtimedf.solidx.iloc[0]
                 self.best_soltime = best_rocb_time
+                self.best_splitK = self.rocb_gtimedf.splitK.iloc[0]
+                self.best_err_ratio = self.rocb_gtimedf.err_ratio.iloc[0]
+                self.best_kernelName = self.rocb_gtimedf.kernelName.iloc[0]
             else:
                 self.best_libtype = "hipblaslt"
-                self.best_solidx = self.hipb_gtimedf.index[0]
+                self.best_solidx = self.hipb_gtimedf.solidx.iloc[0]
                 self.best_soltime = best_hipb_time
+                self.best_splitK = self.hipb_gtimedf.splitK.iloc[0]
+                self.best_err_ratio = self.hipb_gtimedf.err_ratio.iloc[0]
             # self.check_gemm_ref(self.best_libtype,self.best_solidx)
         elif len(self.hipb_gtimedf) > 0:
-            print(">>> Only hipblas solutions found!", flush=True)
+            print(">>> Only hipblas or asm solutions found!", flush=True)
             best_hipb_time = self.hipb_gtimedf.gtimems.iloc[0]
-            self.best_libtype = "hipblaslt"
-            self.best_solidx = self.hipb_gtimedf.index[0]
+            self.best_libtype = self.hipb_gtimedf.libtype.iloc[0]
+            self.best_solidx = self.hipb_gtimedf.solidx.iloc[0]
             self.best_soltime = best_hipb_time
+            self.best_splitK = self.hipb_gtimedf.splitK.iloc[0]
+            self.best_err_ratio = self.hipb_gtimedf.err_ratio.iloc[0]
+            self.best_kernelName = self.hipb_gtimedf.kernelName.iloc[0]
         elif len(self.rocb_gtimedf) > 0:
             print(">>> Only rocblas solutions found!", flush=True)
             best_rocb_time = self.rocb_gtimedf.gtimems.iloc[0]
-            self.best_libtype = "rocblas"
-            self.best_solidx = self.rocb_gtimedf.index[0]
+            self.best_libtype = self.rocb_gtimedf.libtype.iloc[0]
+            self.best_solidx = self.rocb_gtimedf.solidx.iloc[0]
             self.best_soltime = best_rocb_time
+            self.best_splitK = self.rocb_gtimedf.splitK.iloc[0]
+            self.best_err_ratio = self.rocb_gtimedf.err_ratio.iloc[0]
+            self.best_kernelName = self.rocb_gtimedf.kernelName.iloc[0]
         else:
-            print(">>> No rocblas or hipblas solutions found!", flush=True)
+            print(">>> No rocblas or hipblas or asm solutions found!", flush=True)
             self.best_libtype = "rocblas"
             self.best_solidx = 0
             self.best_soltime = 0
+            self.best_splitK = 0
+            self.best_err_ratio = 0
+            self.best_kernelName = ""
         print(
             ">>> Fastest Solution is",
             self.best_libtype,
             self.best_solidx,
             self.best_soltime,
+            self.best_splitK,
+            self.best_err_ratio,
             flush=True,
         )
 
@@ -444,6 +707,7 @@ class GemmTuner:
         rocblas_decode=False,
         mp=1,
         err_ratio=0.01,
+        profile_file="",
     ):
         self.gemm_problems = pd.DataFrame(columns=["M", "N", "K", "bias"])
         self.indtype = indtype
@@ -451,6 +715,8 @@ class GemmTuner:
         self.rocblas_decode = rocblas_decode
         self.tuned_file = tuned_file
         self.mp = mp
+        self.err_ratio = err_ratio
+        self.profile_file = profile_file
 
         tuned_file_path = Path(tuned_file)
         if tuned_file_path.exists():
@@ -461,7 +727,7 @@ class GemmTuner:
             self.tuned_shapes = None
             with open(tuned_file, "w") as tf:
                 tf.write(
-                    "M,N,K,bias,dtype,outdtype,scaleAB,cu_num,libtype,solidx,soltimes,kernelName,tflops, bw\n"
+                    "M,N,K,bias,dtype,outdtype,scaleAB,cu_num,libtype,solidx,splitK,soltimes,kernelName,err_ratio,tflops,bw\n"
                 )
 
         gpu = torch.cuda.current_device()
@@ -518,18 +784,21 @@ class GemmTuner:
                 scaleAB=ds["scaleAB"],
                 rocblas_decode=self.rocblas_decode,
                 mp=self.mp,
-                err_ratio=0.01,
+                err_ratio=self.err_ratio,
+                profile_file=self.profile_file,
             )
             gemmobj.find_fastest_solution()
 
             soltimes = round(gemmobj.best_soltime * 1000, 2)
+            splitK = gemmobj.best_splitK
+            err_ratio = gemmobj.best_err_ratio
             kernal_name = (
                 aiter.getHipblasltKernelName(int(gemmobj.best_solidx))
                 if gemmobj.best_libtype == "hipblaslt"
-                else "rocblas"
+                else gemmobj.best_kernelName
             )
             ret = (
-                ((self.cu_num, ds["M"], ds["N"], ds["K"]), int(gemmobj.best_solidx)),
+                (self.cu_num, ds["M"], ds["N"], ds["K"]),
                 soltimes,
                 gemmobj.check_err_ratio,
             )
@@ -541,7 +810,7 @@ class GemmTuner:
             with open(self.tuned_file, "a") as tf:
                 tf.write(
                     f"{ds['M']},{ds['N']},{ds['K']},{ds['bias']},{indtype},{outdtype},{ds['scaleAB']},"
-                    f"{self.cu_num},{gemmobj.best_libtype},{int(gemmobj.best_solidx)},{soltimes},{kernal_name},{tflops},{bw}\n"
+                    f"{self.cu_num},{gemmobj.best_libtype},{int(gemmobj.best_solidx)},{int(splitK)},{(soltimes)},{kernal_name},{err_ratio},{tflops},{bw}\n"
                 )
 
             del gemmobj
