@@ -13,4 +13,5 @@ void moe_sorting_fwd(torch::Tensor &topk_ids,              // [m, topk]
                      int num_experts,
                      int unit_size,
                      std::optional<torch::Tensor> local_expert_mask = std::nullopt,
-                     std::optional<torch::Tensor> num_local_tokens = std::nullopt);
\ No newline at end of file
+                     std::optional<torch::Tensor> num_local_tokens = std::nullopt,
+                     int dispatch_policy = 0);
\ No newline at end of file
