@@ -1,18 +1,23 @@
 # SPDX-License-Identifier: MIT
 # Copyright (C) 2024-2025, Advanced Micro Devices, Inc. All rights reserved.
 
+import functools
+import os
+from typing import Optional
+
+import pandas as pd
 import torch
 from torch import Tensor
-from typing import Optional
+
 from aiter import logger
+
 from ..jit.core import (
-    compile_ops,
+    AITER_CONFIG_GEMM_A4W4_FILE,
+    AITER_LOG_TUNED_CONFIG,
     AITER_ROOT_DIR,
+    compile_ops,
 )
-from ..jit.utils.chip_info import get_cu_num
-from ..jit.utils.chip_info import get_gfx
-import functools
-import pandas as pd
+from ..jit.utils.chip_info import get_cu_num, get_gfx
 from ..ops.gemm_op_common import get_padded_m
 
 
@@ -32,9 +37,7 @@ def compute_gemm_SplitK(M: int, N: int, K: int, tile_m: int, tile_n: int, tile_k
 @functools.lru_cache(maxsize=1024)
 def get_GEMM_config(M: int, N: int, K: int):
     if not hasattr(get_GEMM_config, "gemm_dict"):
-        gemm_dict = pd.read_csv(
-            f"{AITER_ROOT_DIR}/aiter/configs/a4w4_blockscale_tuned_gemm.csv"
-        ).drop_duplicates()
+        gemm_dict = pd.read_csv(AITER_CONFIG_GEMM_A4W4_FILE).drop_duplicates()
         get_GEMM_config.gemm_dict = gemm_dict.set_index(
             ["cu_num", "M", "N", "K"]
         ).to_dict("index")
@@ -45,11 +48,15 @@ def get_GEMM_config(M: int, N: int, K: int):
         padded_M = M if gl is None else get_padded_m(M, N, K, gl)
         config = get_GEMM_config.gemm_dict.get((cu_num, padded_M, N, K), None)
         if config is not None:
-            logger.info(
-                f"shape is M:{M}, N:{N}, K:{K}, found padded_M: {padded_M}, N:{N}, K:{K} is tuned on cu_num = {cu_num} in CKGEMM or asmGEMM, kernel name is {config['kernelName']}, splitK is {config['splitK']}!"
-            )
+            if AITER_LOG_TUNED_CONFIG:
+                logger.info(
+                    f"shape is M:{M}, N:{N}, K:{K}, found padded_M: {padded_M}, N:{N}, K:{K} is tuned on cu_num = {cu_num} in {AITER_CONFIG_GEMM_A4W4_FILE}, kernel name is {config['kernelName']}, splitK is {config['splitK']}!"
+                )
             break
-
+    else:
+        logger.info(
+            f"shape is M:{M}, N:{N}, K:{K}, not found tuned config in CKGEMM or asmGEMM, will use default config!"
+        )
     return config
 
 
@@ -86,8 +93,8 @@ def gemm_a4w4(
         splitK = ck_config.get("splitK", None)
         kernelName = ck_config["kernelName"]
     if (
-        m < 256
-        or (ck_config is not None and kernelName.find("_ZN") == -1)
+        ck_config is not None
+        and kernelName.find("_ZN") == -1
         # or bias is None
     ):
         splitK = 0 if splitK is None else splitK
