@@ -10,6 +10,11 @@ from aiter import logger
 from ..jit.core import (
     compile_ops,
     AITER_ROOT_DIR,
+    AITER_CONFIG_GEMM_A8W8_FILE,
+    AITER_CONFIG_GEMM_A8W8_BLOCKSCALE_FILE,
+    AITER_CONFIG_GEMM_A8W8_BPRESHUFFLE_FILE,
+    AITER_CONFIG_GEMM_A8W8_BLOCKSCALE_BPRESHUFFLE_FILE,
+    AITER_LOG_TUNED_CONFIG,
 )
 from ..jit.utils.torch_guard import torch_compile_guard
 from ..utility import dtypes
@@ -217,9 +222,7 @@ def get_CKGEMM_config_(tuned_file: str = None) -> None:
     if _CKGEMM_CONFIG_CACHE is None:
         _CKGEMM_CONFIG_CACHE = {}
     if tuned_file not in _CKGEMM_CONFIG_CACHE:
-        ckgemm_dict = pd.read_csv(
-            f"{AITER_ROOT_DIR}/aiter/configs/{tuned_file}"
-        ).drop_duplicates()
+        ckgemm_dict = pd.read_csv(f"{tuned_file}").drop_duplicates()
         _CKGEMM_CONFIG_CACHE[tuned_file] = ckgemm_dict.set_index(
             ["cu_num", "M", "N", "K"]
         ).to_dict("index")
@@ -239,13 +242,14 @@ def get_CKGEMM_config(M: int, N: int, K: int, tuned_file="a8w8_tuned_gemm.csv"):
         padded_M = M if gl is None else get_padded_m(M, N, K, gl)
         config = _CKGEMM_CONFIG_CACHE[tuned_file].get((cu_num, padded_M, N, K), None)
         if config is not None:
-            logger.info(
-                f"shape is M:{M}, N:{N}, K:{K}, found padded_M: {padded_M}, N:{N}, K:{K} is tuned on cu_num = {cu_num} in CKGEMM , kernel name is {config['kernelName']}!"
-            )
+            if AITER_LOG_TUNED_CONFIG:
+                logger.info(
+                    f"shape is M:{M}, N:{N}, K:{K}, found padded_M: {padded_M}, N:{N}, K:{K} is tuned on cu_num = {cu_num} in {tuned_file} , kernel name is {config['kernelName']}!"
+                )
             break
     if config is None:
         logger.info(
-            f"shape is M:{M}, N:{N}, K:{K}, not found tuned config in CKGEMM, will use default config!"
+            f"shape is M:{M}, N:{N}, K:{K}, not found tuned config in {tuned_file}, will use default config!"
         )
     return config
 
@@ -353,7 +357,7 @@ def gemm_a8w8_CK(
     m = XQ.shape[0]
     n = WQ.shape[0]
     k = XQ.shape[-1]
-    ck_config = get_CKGEMM_config(m, n, k)
+    ck_config = get_CKGEMM_config(m, n, k, AITER_CONFIG_GEMM_A8W8_FILE)
     if splitK is None:
         if ck_config is not None:
             splitK = ck_config["splitK"]
@@ -380,7 +384,7 @@ def gemm_a8w8_bpreshuffle(
     n = WQ.shape[0]
     k = XQ.shape[-1]
 
-    get_CKGEMM_config(m, n, k, "a8w8_bpreshuffle_tuned_gemm.csv")
+    get_CKGEMM_config(m, n, k, AITER_CONFIG_GEMM_A8W8_BPRESHUFFLE_FILE)
     # if (
     #     ck_config is None
     #     and dtype == dtypes.bf16
@@ -420,7 +424,7 @@ def gemm_a8w8_blockscale(
         else:
             assert 0, f"asm kernel only support B preshuffle and m >= 16"
     else:
-        get_CKGEMM_config(m, n, k, "a8w8_blockscale_tuned_gemm.csv")
+        get_CKGEMM_config(m, n, k, AITER_CONFIG_GEMM_A8W8_BLOCKSCALE_FILE)
         return gemm_a8w8_blockscale_ck(XQ, WQ, x_scale, w_scale, Y)
 
 
@@ -451,7 +455,7 @@ def gemm_a8w8_blockscale_bpreshuffle(
     m = XQ.shape[0]
     n = WQ.shape[0]
     k = XQ.shape[1]
-    get_CKGEMM_config(m, n, k, "a8w8_blockscale_bpreshuffle_tuned_gemm.csv")
+    get_CKGEMM_config(m, n, k, AITER_CONFIG_GEMM_A8W8_BLOCKSCALE_BPRESHUFFLE_FILE)
     Y = torch.empty(m, n, dtype=dtype, device=XQ.device)
     return gemm_a8w8_blockscale_bpreshuffle_ck(XQ, WQ, x_scale, w_scale, Y)
 
