@@ -2,8 +2,8 @@
 # Copyright (C) 2024-2025, Advanced Micro Devices, Inc. All rights reserved.
 
 from torch import Tensor, Generator
-from typing import List, Optional, Tuple, Any
-from ..jit.core import compile_ops, CK_DIR, AITER_CSRC_DIR, logger
+from typing import Optional, Tuple, Any
+from ..jit.core import compile_ops, CK_DIR, AITER_CSRC_DIR
 from ..jit.utils.chip_info import get_gfx
 from ..jit.utils.torch_guard import torch_compile_guard
 from ..utility import dtypes
@@ -21,6 +21,8 @@ def cmdGenFunc_mha_fwd(
     window_size_right: int,
     return_softmax_lse: bool,
     return_dropout_randval: bool,
+    cu_seqlens_q: Optional[torch.Tensor] = None,
+    cu_seqlens_kv: Optional[torch.Tensor] = None,
     out: Optional[Tensor] = None,
     bias: Optional[Tensor] = None,
     alibi_slopes: Optional[Tensor] = None,
@@ -40,6 +42,10 @@ def cmdGenFunc_mha_fwd(
     elif q.dtype == dtypes.bf16:
         md_name += "_bf16"
         filter += "bf16*"
+    elif q.dtype == dtypes.fp8:
+        # only support bf16 out for fp8 input
+        md_name += "_fp8bf16"
+        filter += "fp8bf16*"
     if bias is not None:
         md_name += "_bias"
         filter += "_bias*"
@@ -145,6 +151,8 @@ def gen_mha_fwd_fake_tensors(
     window_size_right: int,
     return_softmax_lse: bool,
     return_dropout_randval: bool,
+    cu_seqlens_q: Optional[torch.Tensor] = None,
+    cu_seqlens_kv: Optional[torch.Tensor] = None,
     out: Optional[torch.Tensor] = None,
     bias: Optional[torch.Tensor] = None,
     alibi_slopes: Optional[torch.Tensor] = None,
@@ -172,11 +180,13 @@ def mha_fwd(
     window_size_right: int,
     return_softmax_lse: bool,
     return_dropout_randval: bool,
+    cu_seqlens_q: Optional[torch.Tensor] = None,
+    cu_seqlens_kv: Optional[torch.Tensor] = None,
     out: Optional[Tensor] = None,
     bias: Optional[Tensor] = None,
     alibi_slopes: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]: ...
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def gen_fmha_v3_fwd_fake_tensors(
@@ -218,7 +228,7 @@ def fmha_v3_fwd(
     bias: Optional[Tensor] = None,
     alibi_slopes: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]: ...
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def cmdGenFunc_mha_varlen_fwd(
@@ -244,6 +254,8 @@ def cmdGenFunc_mha_varlen_fwd(
     bias: Optional[torch.Tensor] = None,
     alibi_slopes: Optional[torch.Tensor] = None,
     gen: Optional[torch.Generator] = None,
+    cu_seqlens_q_padded: Optional[torch.Tensor] = None,
+    cu_seqlens_k_padded: Optional[torch.Tensor] = None,
 ):
     # causal=true is the same as causal=false in this case
     causal = is_causal
@@ -258,6 +270,10 @@ def cmdGenFunc_mha_varlen_fwd(
         elif q.dtype == dtypes.bf16:
             md_name += "_bf16"
             filter_fwd += "bf16*"
+        elif q.dtype == dtypes.fp8:
+            # only support bf16 out for fp8 input
+            md_name += "_fp8bf16"
+            filter_fwd += "fp8bf16*"
         if 0.0 < logits_soft_cap:
             md_name += "_logits"
             filter_fwd += "_logits*"
@@ -391,7 +407,9 @@ def gen_mha_varlen_fwd_fake_tensor(
     bias: Optional[torch.Tensor] = None,
     alibi_slopes: Optional[torch.Tensor] = None,
     gen: Optional[torch.Generator] = None,
-) -> List[torch.Tensor]:
+    cu_seqlens_q_padded: Optional[torch.Tensor] = None,
+    cu_seqlens_k_padded: Optional[torch.Tensor] = None,
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
     device = q.device
     dtype = q.dtype
 
@@ -453,7 +471,9 @@ def mha_varlen_fwd(
     bias: Optional[torch.Tensor] = None,
     alibi_slopes: Optional[torch.Tensor] = None,
     gen: Optional[torch.Generator] = None,
-) -> List[torch.Tensor]: ...
+    cu_seqlens_q_padded: Optional[torch.Tensor] = None,
+    cu_seqlens_k_padded: Optional[torch.Tensor] = None,
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def gen_fmha_v3_varlen_fwd_fake_tensor(
@@ -480,7 +500,7 @@ def gen_fmha_v3_varlen_fwd_fake_tensor(
     bias: Optional[torch.Tensor] = None,
     alibi_slopes: Optional[torch.Tensor] = None,
     gen: Optional[torch.Generator] = None,
-) -> List[torch.Tensor]:
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
 
     device = q.device
     dtype = q.dtype
@@ -543,7 +563,7 @@ def fmha_v3_varlen_fwd(
     bias: Optional[torch.Tensor] = None,
     alibi_slopes: Optional[torch.Tensor] = None,
     gen: Optional[torch.Generator] = None,
-) -> List[torch.Tensor]: ...
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def cmdGenFunc_mha_bwd(
@@ -716,7 +736,7 @@ def gen_mha_bwd_fake_tensors(
     alibi_slopes: Optional[Tensor] = None,
     rng_state: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]:
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
     return common_mha_bwd_fake_tensors(q, k, v, dq, dk, dv)
 
 
@@ -747,7 +767,7 @@ def mha_bwd(
     alibi_slopes: Optional[Tensor] = None,
     rng_state: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]: ...
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def gen_fmha_v3_bwd_fake_tensors(
@@ -771,7 +791,7 @@ def gen_fmha_v3_bwd_fake_tensors(
     alibi_slopes: Optional[Tensor] = None,
     rng_state: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]:
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
     return common_mha_bwd_fake_tensors(q, k, v, dq, dk, dv)
 
 
@@ -799,7 +819,7 @@ def fmha_v3_bwd(
     alibi_slopes: Optional[Tensor] = None,
     rng_state: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]: ...
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def cmdGenFunc_mha_varlen_bwd(
@@ -957,11 +977,83 @@ def cmdGenFunc_mha_batch_prefill(
     }
 
 
+def gen_mha_varlen_bwd_fake_tensors_common(
+    q: Tensor,
+    k: Tensor,
+    v: Tensor,
+    cu_seqlens_q: Tensor,
+    max_seqlen_q: int,
+    zero_tensors: bool,
+    dq: Optional[Tensor] = None,
+    dk: Optional[Tensor] = None,
+    dv: Optional[Tensor] = None,
+):
+    num_heads = q.size(1)
+
+    batch_size = cu_seqlens_q.numel() - 1
+    dq_ = torch.empty_like(q)
+    dk_ = torch.empty_like(k)
+    dv_ = torch.empty_like(v)
+    if dq is not None:
+        dq_ = dq
+    else:
+        dq_ = torch.empty_like(q)
+
+    if dk is not None:
+        dk_ = dk
+    else:
+        dk_ = torch.empty_like(k)
+
+    if dv is not None:
+        dv_ = dv
+    else:
+        dv_ = torch.empty_like(v)
+
+    softmax_d = torch.empty(batch_size, num_heads, max_seqlen_q, dtype=torch.float)
+
+    if zero_tensors:
+        dq_.zero_()
+        softmax_d.zero_()
+
+    return dq_, dk_, dv_, softmax_d
+
+
+def gen_mha_varlen_bwd_fake_tensors(
+    dout: Tensor,
+    q: Tensor,
+    k: Tensor,
+    v: Tensor,
+    out: Tensor,
+    softmax_lse: Tensor,
+    cu_seqlens_q: Tensor,
+    cu_seqlens_k: Tensor,
+    max_seqlen_q: int,
+    max_seqlen_k: int,
+    dropout_p: float,
+    softmax_scale: float,
+    zero_tensors: bool,
+    is_causal: bool,
+    window_size_left: int,
+    window_size_right: int,
+    deterministic: bool,
+    dq: Optional[Tensor] = None,
+    dk: Optional[Tensor] = None,
+    dv: Optional[Tensor] = None,
+    alibi_slopes: Optional[Tensor] = None,
+    rng_state: Optional[Tensor] = None,
+    gen: Optional[Generator] = None,
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
+    return gen_mha_varlen_bwd_fake_tensors_common(
+        q, k, v, cu_seqlens_q, max_seqlen_q, zero_tensors, dq, dk, dv
+    )
+    # return common_mha_bwd_fake_tensors(q, k, v, dq, dk, dv)
+
+
 @compile_ops(
     "module_mha_varlen_bwd",
     fc_name="mha_varlen_bwd",
     gen_func=cmdGenFunc_mha_varlen_bwd,
-    gen_fake=gen_mha_bwd_fake_tensors,
+    gen_fake=gen_mha_varlen_bwd_fake_tensors,
 )
 def mha_varlen_bwd(
     dout: Tensor,
@@ -987,7 +1079,7 @@ def mha_varlen_bwd(
     alibi_slopes: Optional[Tensor] = None,
     rng_state: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]: ...
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def gen_fmha_v3_varlen_bwd_fake_tensor(
@@ -1017,7 +1109,10 @@ def gen_fmha_v3_varlen_bwd_fake_tensor(
     rng_state: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
 ):
-    return common_mha_bwd_fake_tensors(q, k, v, dq, dk, dv)
+    return gen_mha_varlen_bwd_fake_tensors_common(
+        q, k, v, cu_seqlens_q, max_seqlen_q, zero_tensors, dq, dk, dv
+    )
+    # return common_mha_bwd_fake_tensors(q, k, v, dq, dk, dv)
 
 
 @compile_ops(
@@ -1053,16 +1148,11 @@ def fmha_v3_varlen_bwd(
     alibi_slopes: Optional[Tensor] = None,
     rng_state: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]: ...
-
-
-@torch_compile_guard()
-def maybe_contiguous_custom_op(x: torch.Tensor) -> torch.Tensor:
-    return x.contiguous() if x is not None and x.stride(-1) != 1 else x
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def maybe_contiguous(x):
-    return maybe_contiguous_custom_op(x)
+    return x.contiguous() if x is not None and x.stride(-1) != 1 else x
 
 
 def _flash_attn_forward(
@@ -1078,6 +1168,8 @@ def _flash_attn_forward(
     alibi_slopes: Optional[torch.Tensor],
     return_lse: bool,
     return_softmax: bool,
+    cu_seqlens_q: Optional[torch.Tensor] = None,
+    cu_seqlens_kv: Optional[torch.Tensor] = None,
 ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
 
     (_, seqlen_q, nhead_q, hdim_q) = q.shape
@@ -1092,19 +1184,32 @@ def _flash_attn_forward(
 
     def can_impl_fmha_v3_fwd():
         # basic
-        gfx = get_gfx()
         ret = alibi_slopes is None
         ret = ret and (bias is None)
         ret = ret and (dropout_p == 0.0)
-        ret = ret and (hdim_q == hdim_v)
-        ret = ret and (hdim_q == 128)
+        ret = ret and (hdim_v == 128)
+        ret = ret and (hdim_q == 128 or (get_gfx() == "gfx950" and hdim_q == 192))
         ret = ret and (nhead_q % nhead_k == 0)
         ret = ret and (not swa)
         ret = ret and (q.dtype == dtypes.bf16)
+        ret = ret and (cu_seqlens_q is None and cu_seqlens_kv is None)
         return ret
 
     q, k, v = [maybe_contiguous(x) for x in (q, k, v)]
-    if can_impl_fmha_v3_fwd():
+
+    # Validate newly added optional cumulative length / padded arrays if provided.
+    # They are currently only plumbed through for future CK support enabling per-batch padding.
+    def _validate_cu(name: str, x: Optional[torch.Tensor]):
+        if x is None:
+            return
+        assert x.dim() == 1, f"{name} must be 1D"
+        assert x.dtype in (torch.int32, torch.int64), f"{name} must be int32/int64"
+        # Lightweight monotonicity / length check deferred until integration point.
+
+    _validate_cu("cu_seqlens_q", cu_seqlens_q)
+    _validate_cu("cu_seqlens_kv", cu_seqlens_kv)
+
+    if can_impl_fmha_v3_fwd() and seqlen_q > 128:  # Prefer CK for decode cases
         out, softmax_lse, S_dmask, rng_state = fmha_v3_fwd(
             q,
             k,
@@ -1133,6 +1238,8 @@ def _flash_attn_forward(
             window_size_right,
             return_lse,
             return_softmax,
+            cu_seqlens_q,
+            cu_seqlens_kv,
             None,
             bias,
             alibi_slopes,
@@ -1160,9 +1267,9 @@ def can_impl_fmha_v3_bwd(
     deterministic: bool,
     is_v3_atomic_fp32: Optional[bool] = True,
 ) -> bool:
+
     (_, seqlen_q, nhead_q, hdim_q) = q.shape
     (_, seqlen_k, nhead_k, hdim_v) = v.shape
-
     batch_stride_q = q.stride(0)
     stride_q = q.stride(1)
     nhead_stride_q = q.stride(2)
@@ -1231,10 +1338,9 @@ def can_impl_fmha_v3_bwd(
         npssk &= (batch_stride_dv / batch_stride_v) == (nhead_q / nhead_k)
 
         hd128_case = (hdim_q == 128) and npssk
-
         hd64_case = (hdim_q == 64 and is_v3_atomic_fp32 == False) and npssk
-
         ret = hd128_case or hd64_case
+        ret &= not swa
 
         return ret
 
@@ -1251,14 +1357,9 @@ def can_impl_fmha_v3_bwd(
         # bwd_hd64_bf16_causal_a32_rtz_pssk
         # bwd_hd64_fp16_a32_pssk
         # bwd_hd64_fp16_causal_a32_pssk
-        gfx = get_gfx()
         # nhead_stride_dq_acc >= stride_dq_acc must be guaranteed
-        ret = (hdim_q == 64 and gfx == "gfx942" and is_v3_atomic_fp32 == True) or (
-            hdim_q == 128 and gfx == "gfx950"
-        )
-        ret &= nmask or (
-            mask and seqlen_q == seqlen_k
-        )  # TODO: or (seqlen_q != seqlen_k and mask_type == top_left)
+        ret = hdim_q == 64 and is_v3_atomic_fp32 == True
+        ret &= not swa
 
         return ret
 
@@ -1286,6 +1387,7 @@ def can_impl_fmha_v3_bwd(
         ret &= nhead_stride_v == nhead_stride_dv
         ret &= (batch_stride_dk / batch_stride_k) == (nhead_q / nhead_k)
         ret &= (batch_stride_dv / batch_stride_v) == (nhead_q / nhead_k)
+        ret &= not swa
 
         return ret
 
@@ -1309,11 +1411,7 @@ def can_impl_fmha_v3_bwd(
         # bwd_hd192_bf16_causal_a32_rtz_psskddv
         ret = is_v3_atomic_fp32 == True
         ret &= hdim_q > 64 and hdim_q <= 192
-        ret &= (
-            nmask
-            or (mask and seqlen_q == seqlen_k)
-            or (swa and hdim_q > 64 and hdim_q <= 128)
-        )  # TODO: or (seqlen_q != seqlen_k and mask_type == top_left)
+        ret &= nmask or mask or (swa and hdim_q > 64 and hdim_q <= 128)
 
         return ret
 
@@ -1353,11 +1451,10 @@ def _flash_attn_backward(
     is_v3_atomic_fp32: Optional[bool] = True,
     how_v3_bf16_cvt: Optional[int] = 1,
 ) -> torch.Tensor:
+    # rtna & rtz are deprecated in gfx950
     if get_gfx() == "gfx950" and how_v3_bf16_cvt != 0:
-        logger.warning(
-            "Rounding mode RTNA & RTZ are deprecated in gfx950, ignore option `how_v3_bf16_cvt`"
-        )
         how_v3_bf16_cvt = 0
+
     # can_impl_fmha_v3_bwd should before maybe_contiguous to get pure dout, q, k, v, out
     can_impl_fmha_v3_bwd_ = can_impl_fmha_v3_bwd(
         dout,
@@ -1379,7 +1476,43 @@ def _flash_attn_backward(
 
     # dq, dk, dv are allocated by us so they should already be contiguous
     dout, q, k, v, out = [maybe_contiguous(x) for x in (dout, q, k, v, out)]
-    if can_impl_fmha_v3_bwd_:
+
+    (_, seqlen_q, nhead_q, hdim_q) = q.shape
+    (_, seqlen_k, nhead_k, hdim_v) = v.shape
+    mask = causal and window_size_left == -1  # causal mask
+    nmask = not causal and window_size_left == -1 and window_size_right == -1  # no mask
+    swa = (window_size_left > 0) or (window_size_right > 0)
+
+    # only 1 block when sk <= 256, thus deterministic
+    is_950_1block = (
+        get_gfx() == "gfx950"
+        and seqlen_k <= 256
+        and hdim_q > 64
+        and hdim_q <= 128
+        and hdim_q % 8 == 0
+        and not swa
+    )
+
+    def can_impl_fmha_v3_bwd_gfx950():
+        ret = get_gfx() == "gfx950"
+        ret &= alibi_slopes is None
+        ret &= bias is None
+        ret &= dbias is None
+        ret &= dropout_p == 0.0
+        ret &= not deterministic or is_950_1block
+        ret &= hdim_q == hdim_v
+        ret &= nhead_q % nhead_k == 0
+        ret &= hdim_q > 64 and hdim_q <= 128 and hdim_q % 8 == 0
+
+        return ret
+
+    can_impl_fmha_v3_bwd_ |= can_impl_fmha_v3_bwd_gfx950()
+
+    if (
+        can_impl_fmha_v3_bwd_ and seqlen_q > 16
+    ):  # ck fmha bwd has optimization for seqlen_q <= 16
+        if dq is not None:
+            dq.zero_()
         (
             dq,
             dk,
@@ -1397,8 +1530,8 @@ def _flash_attn_backward(
             causal,
             window_size_left,
             window_size_right,
-            deterministic,
-            is_v3_atomic_fp32,
+            False if is_950_1block else deterministic,
+            False if is_950_1block else is_v3_atomic_fp32,
             how_v3_bf16_cvt,
             dq,
             dk,
@@ -1458,6 +1591,8 @@ class FlashAttnFunc(torch.autograd.Function):
         is_grad_enabled,
         is_v3_atomic_fp32: Optional[bool] = True,
         how_v3_bf16_cvt: Optional[int] = 1,
+        cu_seqlens_q: Optional[torch.Tensor] = None,
+        cu_seqlens_kv: Optional[torch.Tensor] = None,
     ):
         is_grad = is_grad_enabled and any(x.requires_grad for x in [q, k, v])
         if softmax_scale is None:
@@ -1482,6 +1617,8 @@ class FlashAttnFunc(torch.autograd.Function):
             alibi_slopes=alibi_slopes,
             return_lse=return_lse,
             return_softmax=return_softmax and dropout_p > 0,
+            cu_seqlens_q=cu_seqlens_q,
+            cu_seqlens_kv=cu_seqlens_kv,
         )
         if is_grad:
             assert return_lse
@@ -1509,7 +1646,7 @@ class FlashAttnFunc(torch.autograd.Function):
     @staticmethod
     def backward(ctx, dout, *args):
         q, k, v, out, softmax_lse, rng_state = ctx.saved_tensors
-        dq, dk, dv = torch.zeros_like(q), torch.empty_like(k), torch.empty_like(v)
+        dq, dk, dv = torch.empty_like(q), torch.empty_like(k), torch.empty_like(v)
         bias = ctx.bias
         dbias = torch.empty_like(bias) if bias is not None else None
         head_size_q_og = ctx.head_size_q_og
@@ -1543,7 +1680,44 @@ class FlashAttnFunc(torch.autograd.Function):
         dq = dq[..., :head_size_q_og]  # We could have padded the head dimension
         dk = dk[..., :head_size_q_og]
         dv = dv[..., :head_size_v_og]
-        return dq, dk, dv, None, None, None, None, dbias, None, None, None, None, None
+        # Forward positional args order:
+        #  1 q
+        #  2 k
+        #  3 v
+        #  4 dropout_p
+        #  5 softmax_scale
+        #  6 causal
+        #  7 window_size (tuple - no grad)
+        #  8 bias
+        #  9 alibi_slopes
+        # 10 deterministic
+        # 11 return_lse
+        # 12 return_softmax
+        # 13 is_grad_enabled
+        # 14 is_v3_atomic_fp32
+        # 15 how_v3_bf16_cvt
+        # 16 cu_seqlens_q
+        # 17 cu_seqlens_kv
+        # Need to return exactly 17 gradient entries.
+        return (
+            dq,  # q
+            dk,  # k
+            dv,  # v
+            None,  # dropout_p
+            None,  # softmax_scale
+            None,  # causal
+            None,  # window_size
+            dbias,  # bias
+            None,  # alibi_slopes
+            None,  # deterministic
+            None,  # return_lse
+            None,  # return_softmax
+            None,  # is_grad_enabled
+            None,  # is_v3_atomic_fp32
+            None,  # how_v3_bf16_cvt
+            None,  # cu_seqlens_q
+            None,  # cu_seqlens_kv
+        )
 
 
 def flash_attn_func(
@@ -1559,6 +1733,8 @@ def flash_attn_func(
     deterministic=True,
     return_lse=False,
     return_attn_probs=False,
+    cu_seqlens_q: Optional[torch.Tensor] = None,
+    cu_seqlens_kv: Optional[torch.Tensor] = None,
 ):
     """dropout_p should be set to 0.0 during evaluation
     Supports multi-query and grouped-query attention (MQA/GQA) by passing in KV with fewer heads
@@ -1600,6 +1776,8 @@ def flash_attn_func(
         return_attn_probs: bool. Whether to return the attention probabilities. This option is for
            testing only. The returned probabilities are not guaranteed to be correct
            (they might not have the right scaling).
+        cu_seqlens_q: (batch_size + 1,). The cumulative sequence lengths of the query sequences.
+        cu_seqlens_kv: (batch_size + 1,). The cumulative sequence lengths of the key/value sequences.
     Return:
         out: (batch_size, seqlen, nheads, headdim_v).
         softmax_lse [optional, if return_attn_probs=True]: (batch_size, nheads, seqlen). The
@@ -1623,6 +1801,10 @@ def flash_attn_func(
         return_lse,
         return_attn_probs,
         torch.is_grad_enabled(),
+        True,  # is_v3_atomic_fp32
+        1,  # how_v3_bf16_cvt
+        cu_seqlens_q,
+        cu_seqlens_kv,
     )
 
 
@@ -1632,6 +1814,8 @@ def _flash_attn_varlen_forward(
     v: torch.Tensor,
     cu_seqlens_q: torch.Tensor,
     cu_seqlens_k: torch.Tensor,
+    cu_seqlens_q_padded: Optional[torch.Tensor],
+    cu_seqlens_k_padded: Optional[torch.Tensor],
     max_seqlen_q: int,
     max_seqlen_k: int,
     min_seqlen_q: int,
@@ -1666,15 +1850,15 @@ def _flash_attn_varlen_forward(
 
     def can_impl_fmha_v3_fwd():
         # basic
-        gfx = get_gfx()
         ret = alibi_slopes is None
         ret = ret and (bias is None)
         ret = ret and (dropout_p == 0.0)
-        ret = ret and (hdim_q == hdim_v)
-        ret = ret and (hdim_q == 128)
+        ret = ret and (hdim_v == 128)
+        ret = ret and (hdim_q == 128 or (get_gfx() == "gfx950" and hdim_q == 192))
         ret = ret and (nhead_q % nhead_k == 0)
         ret = ret and (not swa)
         ret = ret and (q.dtype == dtypes.bf16)
+        ret = ret and (cu_seqlens_q_padded is None and cu_seqlens_k_padded is None)
         return ret
 
     q, k, v = [maybe_contiguous(x) for x in (q, k, v)]
@@ -1707,6 +1891,23 @@ def _flash_attn_varlen_forward(
             # custom_build_args={"md_name": md_name, "blob_gen_cmd": blob_gen_cmd},
         )
     else:
+        # Input validation for padded cumulative arrays if provided
+        def _validate(name: str, t: torch.Tensor):
+            assert t.dim() == 1, f"{name} must be 1D"
+            assert t.is_cuda, f"{name} must be on CUDA"
+            assert t.dtype == torch.int32, f"{name} must be int32, actual: {t.dtype}"
+            assert t.is_contiguous(), f"{name} must be contiguous"
+            assert (
+                t.numel() == cu_seqlens_q.numel()
+            ), f"{name} length mismatch with batch"
+            # light monotonic check (first and last only; deeper check in C++)
+            assert t[0].item() == 0, f"{name}[0] must be 0"
+
+        if cu_seqlens_q_padded is not None:
+            _validate("cu_seqlens_q_padded", cu_seqlens_q_padded)
+        if cu_seqlens_k_padded is not None:
+            _validate("cu_seqlens_k_padded", cu_seqlens_k_padded)
+
         out, softmax_lse, S_dmask, rng_state = mha_varlen_fwd(
             q,
             k,
@@ -1725,12 +1926,13 @@ def _flash_attn_varlen_forward(
             window_size_right,
             return_lse,
             return_softmax,
-            out,
-            block_table,
-            bias,
-            alibi_slopes,
-            None,
-            # custom_build_args={"md_name": md_name, "blob_gen_cmd": blob_gen_cmd},
+            out=out,
+            block_table=block_table,
+            bias=bias,
+            alibi_slopes=alibi_slopes,
+            gen=None,
+            cu_seqlens_q_padded=cu_seqlens_q_padded,
+            cu_seqlens_k_padded=cu_seqlens_k_padded,
         )
     return out, softmax_lse, S_dmask, rng_state
 
@@ -1778,6 +1980,7 @@ def _flash_attn_varlen_backward(
     nmask = (
         causal == False and window_size_left == -1 and window_size_right == -1
     )  # no mask
+    swa = (window_size_left > 0) or (window_size_right > 0)
 
     def pssk():
         # only for hd64 a32 causal/no causal, fp16/bf16-rtne/rtna/rtz cases
@@ -1804,7 +2007,6 @@ def _flash_attn_varlen_backward(
             is_v3_atomic_fp32 == True
         )  # nhead_stride_dq_acc >= stride_dq_acc must be guaranteed
         ret &= hdim_q == 64 or hdim_q == 128
-        ret &= nmask  # TODO: or (mask and mask_type == mask_enum::mask_top_left)
 
         return ret
 
@@ -1821,7 +2023,6 @@ def _flash_attn_varlen_backward(
             is_v3_atomic_fp32 == True
         )  # nhead_stride_dq_acc >= stride_dq_acc must be guaranteed
         ret &= hdim_q >= 64 and hdim_q <= 192
-        ret &= nmask  # TODO: or (mask and mask_type == mask_enum::mask_top_left)
 
         return ret
 
@@ -1834,15 +2035,31 @@ def _flash_attn_varlen_backward(
         ret &= deterministic == False
         ret &= hdim_q == hdim_v
         ret &= nhead_q % nhead_k == 0
-        ret &= hdim_q >= 64 and hdim_q <= 128 and hdim_q % 8 == 0
-        ret &= mask or nmask
+        ret &= hdim_q >= 64 and hdim_q <= 192 and hdim_q % 8 == 0
+        ret &= not swa
         ret &= pssk() or psskddv()
 
         return ret
 
+    def can_impl_fmha_v3_bwd_gfx950():
+        ret = get_gfx() == "gfx950"
+        ret &= alibi_slopes is None
+        # ret &= bias is None
+        # ret &= dbias is None
+        ret &= dropout_p == 0.0
+        ret &= deterministic == False
+        ret &= hdim_q == hdim_v
+        ret &= nhead_q % nhead_k == 0
+        ret &= hdim_q > 64 and hdim_q <= 128 and hdim_q % 8 == 0
+        ret &= not swa
+
+        return ret
+
+    can_impl_fmha_v3_bwd_ = can_impl_fmha_v3_bwd() or can_impl_fmha_v3_bwd_gfx950()
     # dq, dk, dv are allocated by us so they should already be contiguous
     dout, q, k, v, out = [maybe_contiguous(x) for x in (dout, q, k, v, out)]
-    if can_impl_fmha_v3_bwd():
+
+    if can_impl_fmha_v3_bwd_:
         (
             dq,
             dk,
@@ -1937,6 +2154,8 @@ class FlashAttnVarlenFunc(torch.autograd.Function):
         block_table,
         out,
         is_grad_enabled,
+        cu_seqlens_q_padded=None,
+        cu_seqlens_k_padded=None,
         is_v3_atomic_fp32: Optional[bool] = True,
         how_v3_bf16_cvt: Optional[int] = 1,
     ):
@@ -1950,12 +2169,15 @@ class FlashAttnVarlenFunc(torch.autograd.Function):
             k = torch.nn.functional.pad(k, [0, 8 - head_size_q_og % 8])
         if head_size_v_og % 8 != 0:
             v = torch.nn.functional.pad(v, [0, 8 - head_size_v_og % 8])
+
         out_padded, softmax_lse, S_dmask, rng_state = _flash_attn_varlen_forward(
             q,
             k,
             v,
             cu_seqlens_q,
             cu_seqlens_k,
+            cu_seqlens_q_padded,
+            cu_seqlens_k_padded,
             max_seqlen_q,
             max_seqlen_k,
             min_seqlen_q,
@@ -1974,6 +2196,7 @@ class FlashAttnVarlenFunc(torch.autograd.Function):
             out=out,
         )
         if is_grad:
+
             assert return_lse
             ctx.save_for_backward(
                 q, k, v, out_padded, softmax_lse, cu_seqlens_q, cu_seqlens_k, rng_state
@@ -2050,28 +2273,50 @@ class FlashAttnVarlenFunc(torch.autograd.Function):
         dq = dq[..., :head_size_q_og]  # We could have padded the head dimension
         dk = dk[..., :head_size_q_og]
         dv = dv[..., :head_size_v_og]
+        # Forward signature (positional args):
+        # q, k, v,
+        # cu_seqlens_q, cu_seqlens_k,
+        # max_seqlen_q, max_seqlen_k,
+        # min_seqlen_q,
+        # dropout_p, softmax_scale, logits_soft_cap,
+        # causal,
+        # window_size,
+        # bias,
+        # alibi_slopes,
+        # deterministic,
+        # return_lse, return_softmax,
+        # block_table,
+        # out,
+        # is_grad_enabled,
+        # cu_seqlens_q_padded, cu_seqlens_k_padded,
+        # is_v3_atomic_fp32, how_v3_bf16_cvt
+        # We only have gradients for q,k,v (dq,dk,dv) and possibly bias (dbias). Others are None.
         return (
-            dq,
-            dk,
-            dv,
-            None,
-            None,
-            None,
-            None,
-            None,
-            None,
-            None,
-            None,
-            None,
-            None,
-            dbias,
-            None,
-            None,
-            None,
-            None,
-            None,
-            None,
-            None,
+            dq,  # q
+            dk,  # k
+            dv,  # v
+            None,  # cu_seqlens_q
+            None,  # cu_seqlens_k
+            None,  # max_seqlen_q
+            None,  # max_seqlen_k
+            None,  # min_seqlen_q
+            None,  # dropout_p
+            None,  # softmax_scale
+            None,  # logits_soft_cap
+            None,  # causal
+            None,  # window_size (tuple treated as arg)
+            dbias,  # bias
+            None,  # alibi_slopes
+            None,  # deterministic
+            None,  # return_lse
+            None,  # return_softmax
+            None,  # block_table
+            None,  # out
+            None,  # is_grad_enabled
+            None,  # cu_seqlens_q_padded
+            None,  # cu_seqlens_k_padded
+            None,  # is_v3_atomic_fp32
+            None,  # how_v3_bf16_cvt
         )
 
 
@@ -2096,7 +2341,16 @@ def flash_attn_varlen_func(
     return_attn_probs=False,
     block_table=None,
     out=None,
+    cu_seqlens_q_padded: Optional[torch.Tensor] = None,
+    cu_seqlens_k_padded: Optional[torch.Tensor] = None,
 ):
+    if block_table is not None and (
+        cu_seqlens_q_padded is not None or cu_seqlens_k_padded is not None
+    ):
+        raise NotImplementedError(
+            "Paged/Split-KV attention (using block_table) does not currently support "
+            "physical sequence padding (cu_seqlens_*_padded)."
+        )
     """dropout_p should be set to 0.0 during evaluation
     Supports multi-query and grouped-query attention (MQA/GQA) by passing in K, V with fewer heads
     than Q. Note that the number of heads in Q must be divisible by the number of heads in KV.
@@ -2175,6 +2429,10 @@ def flash_attn_varlen_func(
         block_table,
         out,
         torch.is_grad_enabled(),
+        cu_seqlens_q_padded,
+        cu_seqlens_k_padded,
+        True,
+        1,
     )
 
 
@@ -2199,7 +2457,7 @@ def mha_batch_prefill_fake_tensors(
     out: Optional[torch.Tensor] = None,
     alibi_slopes: Optional[torch.Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]:
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
     # ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
     num_heads = q.size(1)  # num_heads = q.sizes()[1]
     head_size_v = v.size(2)  # head_size_v = v.size(2)
@@ -2264,7 +2522,7 @@ def mha_batch_prefill(
     out: Optional[Tensor] = None,
     alibi_slopes: Optional[Tensor] = None,
     gen: Optional[Generator] = None,
-) -> List[Tensor]: ...
+) -> Tuple[Tensor, Tensor, Tensor, Tensor]: ...
 
 
 def _mha_batch_prefill(
@@ -2374,3 +2632,87 @@ def mha_batch_prefill_func(
         result.append(S_dmask)
 
     return result[0] if len(result) == 1 else tuple(result)
+
+
+def flash_attn_fp8_pertensor_func(
+    q,
+    k,
+    v,
+    causal=False,
+    window_size=(-1, -1),  # -1 means infinite context window
+    softmax_scale=None,
+):
+    if softmax_scale is None:
+        softmax_scale = q.shape[-1] ** (-0.5)
+    head_size_q_og = q.size(3)
+    head_size_v_og = v.size(3)
+    if head_size_q_og % 8 != 0:
+        q = torch.nn.functional.pad(q, [0, 8 - head_size_q_og % 8])
+        k = torch.nn.functional.pad(k, [0, 8 - head_size_q_og % 8])
+    if head_size_v_og % 8 != 0:
+        v = torch.nn.functional.pad(v, [0, 8 - head_size_v_og % 8])
+    out_padded, _, _, _ = _flash_attn_forward(
+        q,
+        k,
+        v,
+        0.0,
+        softmax_scale,
+        causal=causal,
+        window_size_left=int(window_size[0]),
+        window_size_right=int(window_size[1]),
+        bias=None,
+        alibi_slopes=None,
+        return_lse=False,
+        return_softmax=False,
+    )
+    out = out_padded[..., :head_size_v_og]
+    return out
+
+
+def flash_attn_varlen_fp8_pertensor_func(
+    q,
+    k,
+    v,
+    cu_seqlens_q,
+    cu_seqlens_k,
+    max_seqlen_q,
+    max_seqlen_k,
+    min_seqlen_q=0,
+    logits_soft_cap=0.0,
+    causal=False,
+    window_size=(-1, -1),  # -1 means infinite context window
+    softmax_scale=None,
+):
+    if softmax_scale is None:
+        softmax_scale = q.shape[-1] ** (-0.5)
+    head_size_q_og = q.size(-1)
+    head_size_v_og = v.size(-1)
+    if head_size_q_og % 8 != 0:
+        q = torch.nn.functional.pad(q, [0, 8 - head_size_q_og % 8])
+        k = torch.nn.functional.pad(k, [0, 8 - head_size_q_og % 8])
+    if head_size_v_og % 8 != 0:
+        v = torch.nn.functional.pad(v, [0, 8 - head_size_v_og % 8])
+    out_padded, _, _, _ = _flash_attn_varlen_forward(
+        q,
+        k,
+        v,
+        cu_seqlens_q,
+        cu_seqlens_k,
+        None,
+        None,
+        max_seqlen_q,
+        max_seqlen_k,
+        min_seqlen_q,
+        0.0,
+        softmax_scale,
+        causal=causal,
+        logits_soft_cap=logits_soft_cap,
+        window_size_left=int(window_size[0]),
+        window_size_right=int(window_size[1]),
+        bias=None,
+        alibi_slopes=None,
+        return_lse=False,
+        return_softmax=False,
+    )
+    out = out_padded[..., :head_size_v_og]
+    return out
