@@ -2,21 +2,26 @@
 
 set -ex
 
-# Install dependencies
+echo
+echo "==== ROCm Packages Installed ===="
+dpkg -l | grep rocm || echo "No ROCm packages found."
+
+echo
+echo "==== Install dependencies and aiter ===="
 pip install --upgrade pandas zmq einops numpy==1.26.2
 pip uninstall -y aiter || true
 pip install --upgrade "pybind11>=3.0.1"
 python3 setup.py develop
 
-# Install aiter
+echo
+echo "==== Install triton ===="
 pip uninstall -y triton || true
-
-# Clone the triton repository and install it
 git clone --depth=1 https://github.com/triton-lang/triton || true
 cd triton
 pip install -r python/requirements.txt
 pip install filecheck
-pip install .
+MAX_JOBS=64 pip install .
 
-# Display installed packages
+echo
+echo "==== Show installed packages ===="
 pip list
