@@ -54,7 +54,7 @@ def run_torch(
     else:
         attn_bias = None
 
-    out, _ = attention_ref(
+    out, _, _ = attention_ref(
         q,
         k,
         v,
@@ -94,6 +94,7 @@ def run_ck(
     return_attn_probs=False,
     cu_seqlens_q_padded=None,
     cu_seqlens_k_padded=None,
+    input_layout="BSHD",
 ):
     _, _, nhead, d = q.shape
     _, _, _, d_v = v.shape
@@ -111,7 +112,16 @@ def run_ck(
         output_pad_fn,
         dq_pad_fn,
         dk_pad_fn,
-    ) = generate_qkv(q, k, v, query_padding_mask, key_padding_mask, kvpacked=False)
+    ) = generate_qkv(
+        q,
+        k,
+        v,
+        query_padding_mask,
+        key_padding_mask,
+        kvpacked=(input_layout == "KVPACKED"),
+        qkvpacked=(input_layout == "QKVPACKED"),
+        input_layout=input_layout,
+    )
     batch_size = q.shape[0]
 
     if bias is not None:
@@ -324,6 +334,7 @@ def run_ck_seq_padding(
     return torch.stack(out_batches, dim=0)
 
 
+@pytest.mark.parametrize("input_layout", ["BSHD", "KVPACKED"])
 @pytest.mark.parametrize("dtype", [dtypes.fp16, dtypes.bf16])
 @pytest.mark.parametrize("mha_type", ["mha", "mqa", "gqa"])
 @pytest.mark.parametrize("deterministic", [True, False])
@@ -366,7 +377,6 @@ def run_ck_seq_padding(
         (2048, 2048),
     ],
 )
-@benchmark()
 def test_flash_attn_varlen_func(
     batch_size,
     nheads,
@@ -382,6 +392,7 @@ def test_flash_attn_varlen_func(
     deterministic,
     mha_type,
     dtype,
+    input_layout,
 ):
     return_lse = True
     torch.random.manual_seed(0)
@@ -427,6 +438,10 @@ def test_flash_attn_varlen_func(
             seqlen_k, batch_size, "cuda", mode="random"
         )
 
+    if input_layout == "QKVPACKED":
+        query_padding_mask = None
+        key_padding_mask = None
+
     attn_bias = None
     alibi_slopes = None
     if bias_type == "bias":
@@ -481,6 +496,9 @@ def test_flash_attn_varlen_func(
         deterministic,
         return_lse,
         return_attn_probs,
+        None,
+        None,
+        input_layout,
     )
 
     out_ref, dq_ref, dk_ref, dv_ref = run_torch(
@@ -551,6 +569,43 @@ def test_flash_attn_varlen_func(
     return ret
 
 
+@benchmark()
+def flash_attn_varlen_func_benchmark(
+    batch_size,
+    nheads,
+    seqlen_q,
+    seqlen_k,
+    d,
+    d_v,
+    min_seqlen_q,
+    dropout_p,
+    causal,
+    local,
+    bias_type,
+    deterministic,
+    mha_type,
+    dtype,
+    input_layout,
+):
+    return test_flash_attn_varlen_func(
+        batch_size=batch_size,
+        nheads=nheads,
+        seqlen_q=seqlen_q,
+        seqlen_k=seqlen_k,
+        d=d,
+        d_v=d_v,
+        min_seqlen_q=min_seqlen_q,
+        dropout_p=dropout_p,
+        causal=causal,
+        local=local,
+        bias_type=bias_type,
+        deterministic=deterministic,
+        mha_type=mha_type,
+        dtype=dtype,
+        input_layout=input_layout,
+    )
+
+
 @pytest.mark.parametrize("batch_size", [1, 4])
 @pytest.mark.parametrize("mha_type", ["mha", "mqa", "gqa"])
 @pytest.mark.parametrize("deterministic", [True, False])
@@ -730,9 +785,6 @@ def test_varlen_flash_attn_seq_padding(
     assert out_diff <= out_tol
 
 
-l_dtype = ["bf16", "fp16"]
-l_dim = [32, 40, 64, 111, 128, 160, 192]
-l_mha_type = ["mha", "mqa", "gqa"]
 l_causal = [False, True]
 l_local = [False, True]
 l_deterministic = [False, True]
@@ -770,20 +822,20 @@ if __name__ == "__main__":
     e.g. -s 4,8""",
     )
     parser.add_argument(
-        "-d",
-        type=int,
-        nargs="?",
-        default=None,
-        help="""Dimension of query&key.
-    e.g. -d 128""",
-    )
-    parser.add_argument(
-        "-dv",
-        type=int,
-        nargs="?",
-        default=128,
-        help="""Dimension of value.
-    e.g. -dv 128""",
+        "-d_qk_v",
+        type=dtypes.str2tuple,
+        nargs="+",
+        default=[
+            (32, 32),
+            (40, 40),
+            (64, 64),
+            (111, 111),
+            (128, 128),
+            (160, 160),
+            (192, 192),
+        ],
+        help="""Dimension of query and key. Default is None.
+        e.g.: -qk_v 256,256""",
     )
     parser.add_argument(
         "-dp",
@@ -841,7 +893,9 @@ if __name__ == "__main__":
         "-mha",
         "--mha_type",
         type=str,
-        default=None,
+        nargs="+",
+        choices=["mha", "mqa", "gqa"],
+        default=["mha", "mqa", "gqa"],
         help="""Type of multi-head attention.
     e.g. -mha mha/mqa/gqa""",
     )
@@ -849,51 +903,50 @@ if __name__ == "__main__":
         "-dt",
         "--dtype",
         type=str,
-        default=None,
+        nargs="+",
+        choices=["bf16", "fp16"],
+        default=["bf16", "fp16"],
         help="""Data type.
     e.g.: -dt bf16""",
     )
-
+    parser.add_argument(
+        "-i",
+        "--input_layout",
+        type=str,
+        choices=["BSHD", "KVPACKED"],
+        default="BSHD",
+        help="""input_layout.
+        e.g.: -i BSHD""",
+    )
     args = parser.parse_args()
 
     (seqlen_q, seqlen_k) = args.seqlen_q_k
-    if args.dtype is not None:
-        l_dtype = [dtypes.d_dtypes[args.dtype]]
-    else:
-        l_dtype = [dtypes.d_dtypes[key] for key in l_dtype]
-        args.dtype = "bf16"
-    if args.d is not None:
-        l_dim = [args.d]
-    else:
-        args.d = 128
-    if args.mha_type is not None:
-        l_mha_type = [args.mha_type]
-    else:
-        args.mha_type = "mha"
+
     if args.causal is not None:
         l_causal = [args.causal]
-    else:
-        args.causal = True
     if args.local is not None:
         l_local = [args.local]
-    else:
-        args.local = False
     if args.deterministic is not None:
         l_deterministic = [args.deterministic]
-    else:
-        args.deterministic = True
 
     collected = []
-    for dtype, dim, mha_type, causal, local, deterministic in itertools.product(
-        l_dtype, l_dim, l_mha_type, l_causal, l_local, l_deterministic
+    for (
+        dtype,
+        (dim_qk, dim_v),
+        mha_type,
+        causal,
+        local,
+        deterministic,
+    ) in itertools.product(
+        args.dtype, args.d_qk_v, args.mha_type, l_causal, l_local, l_deterministic
     ):
-        ret = test_flash_attn_varlen_func(
+        ret = flash_attn_varlen_func_benchmark(
             args.batch_size,
             args.nheads,
             seqlen_q,
             seqlen_k,
-            dim,
-            dim,
+            dim_qk,
+            dim_v,
             args.min_seqlen_q,
             args.dropout_p,
             causal,
@@ -901,22 +954,22 @@ if __name__ == "__main__":
             args.bias_type,
             deterministic,
             mha_type,
-            dtype,
+            dtypes.d_dtypes[dtype],
+            args.input_layout,
         )
         collected.append(ret)
+        test_varlen_flash_attn_seq_padding(
+            args.batch_size,
+            mha_type,
+            deterministic,
+            "mixed",
+            dtypes.d_dtypes[dtype],
+            dim_qk,
+            dim_v,
+            seqlen_q,
+            seqlen_k,
+            local,
+        )
 
     df = pd.DataFrame(collected)
     aiter.logger.info(f"mha_varlen summary:\n{df}")
-
-    test_varlen_flash_attn_seq_padding(
-        args.batch_size,
-        args.mha_type,
-        args.deterministic,
-        "mixed",
-        dtype,
-        args.d,
-        args.dv,
-        seqlen_q,
-        seqlen_k,
-        args.local,
-    )
