@@ -1,14 +1,18 @@
 # SPDX-License-Identifier: MIT
 # Copyright (C) 2024-2025, Advanced Micro Devices, Inc. All rights reserved.
 
-import torch
-from typing import List, Optional, Tuple, Union
-import aiter
-from aiter.test_common import checkAllclose, benchmark, perftest
-import random
-import pandas as pd
 import argparse
-from aiter import dtypes
+import itertools
+import random
+from typing import List, Optional, Tuple, Union
+
+import pandas as pd
+import torch
+
+import aiter
+from aiter import dtypes, paged_attn as ops
+from aiter import pertoken_quant
+from aiter.test_common import benchmark, checkAllclose, perftest
 
 torch.set_default_device("cuda")
 torch.set_printoptions(sci_mode=False)
@@ -193,8 +197,6 @@ def pertoken_quant_kvcache_symm(
         .contiguous()
     )
 
-    from aiter import pertoken_quant
-
     k_quant, k_scale_asm = pertoken_quant(k_cache_permute, quant_dtype=quant_dtype)
     v_quant, v_scale_asm = pertoken_quant(v_cache_permute, quant_dtype=quant_dtype)
 
@@ -248,6 +250,75 @@ def run_aiter_asm(
         v_scale,
         None,
         qo_indptr,
+        # kernelName="_ZN5aiter42pa_bf16_pertokenFp8_gqa10_1tg_4w_mtp3_msk1E",
+    )
+
+
+@perftest()
+def run_aiter_hip(
+    query,
+    k_cache,
+    v_cache,
+    block_tables,
+    seq_lens,
+    max_seq_len,
+    max_qlen,
+    kv_cache_dtype,
+    num_kv_heads,
+    scale,
+    k_scale=None,
+    v_scale=None,
+):
+    return ops.PagedAttention.forward_decode(
+        query,
+        k_cache,
+        v_cache,
+        block_tables,
+        seq_lens,
+        max_seq_len,
+        kv_cache_dtype,
+        num_kv_heads,
+        scale,
+        None,
+        k_scale,
+        v_scale,
+        mtp=max_qlen,
+    )
+
+
+@perftest()
+def run_aiter_hip(
+    query,
+    k_cache,
+    v_cache,
+    block_tables,
+    seq_lens,
+    max_seq_len,
+    max_qlen,
+    kv_cache_dtype,
+    num_kv_heads,
+    scale,
+    k_scale=None,
+    v_scale=None,
+    q_scale=None,
+    output_dtype=dtypes.bf16,
+):
+    return aiter.paged_attn.PagedAttention.forward_decode(
+        query,
+        k_cache,
+        v_cache,
+        block_tables,
+        seq_lens,
+        max_seq_len,
+        kv_cache_dtype,
+        num_kv_heads,
+        scale,
+        None,
+        k_scale,
+        v_scale,
+        q_scale=q_scale,
+        mtp=max_qlen,
+        output_dtype=output_dtype,
     )
 
 
@@ -270,11 +341,13 @@ def test_pa_mtp(
     block_size: int,
     dtype: torch.dtype,
     qlen,
-) -> None:
+) -> dict:
+    ret = {}
     seed = 0
     device = "cuda:0"
     torch.set_default_device(device)
     num_query_heads, num_kv_heads = num_heads
+
     assert num_query_heads % num_kv_heads == 0
     max_seq_len = 16384
     max_num_blocks_per_seq = (max_seq_len + block_size - 1) // block_size
@@ -338,26 +411,54 @@ def test_pa_mtp(
         qo_indptr,
     )
 
-    out_asm_noquant, us_asm_noquant = run_aiter_asm(
+    # out_asm_noquant, us_asm_noquant = run_aiter_asm(
+    #     query,
+    #     k_cache,
+    #     asm_V_shuffle(v_cache),
+    #     block_tables,
+    #     seq_lens,
+    #     block_tables.size(1),
+    #     max_qlen,
+    #     qo_indptr=qo_indptr,
+    # )
+    # err_noquant = checkAllclose(
+    #     out_ref_noquant,
+    #     out_asm_noquant,
+    #     msg=f"[torch vs  aiter_asm][No Quant]: {us_asm_noquant:>8.2f} us......",
+    # )
+    # ret["us_asm_bf16"] = us_asm_noquant
+    # ret["err_asm_bf16"] = err_noquant
+
+    scale = float(1.0 / (head_size**0.5))
+    out_hip_noquant, us_hip = run_aiter_hip(
         query,
         k_cache,
-        asm_V_shuffle(v_cache),
+        v_cache,
         block_tables,
         seq_lens,
-        block_tables.size(1),
+        ctx_lens,
         max_qlen,
-        qo_indptr=qo_indptr,
+        "auto",
+        num_kv_heads,
+        scale,
     )
     err_noquant = checkAllclose(
         out_ref_noquant,
-        out_asm_noquant,
-        msg=f"[torch vs  aiter_asm][No Quant]: {us_asm_noquant:>8.2f} us......",
+        out_hip_noquant,
+        msg=f"[torch vs  aiter_hip][No Quant]: {us_hip:>8.2f} us......",
     )
+    ret["us_hip_bf16"] = us_hip
+    ret["err_hip_bf16"] = err_noquant
 
+    # ################## quant start ######################
     k_quant_, k_scale_, v_quant_, v_scale_, k_scale_asm, v_scale_asm = (
         pertoken_quant_kvcache_symm(k_cache, v_cache, quant_dtype=aiter.dtypes.fp8)
     )
 
+    # torch ref
+    out_ref = torch_mha_extend(
+        query, k_quant_, v_quant_, block_tables, seq_lens, qo_indptr, k_scale_, v_scale_
+    )
     out_aiter_asm, us_aiter_asm = run_aiter_asm(
         query,
         k_quant_,
@@ -370,33 +471,50 @@ def test_pa_mtp(
         v_scale_asm,
         qo_indptr,
     )
-
-    out_ref = torch_mha_extend(
-        query, k_quant_, v_quant_, block_tables, seq_lens, qo_indptr, k_scale_, v_scale_
-    )
-
-    # print(out_ref)
-    # print(out_aiter_asm)
-
     err = checkAllclose(
         out_ref,
         out_aiter_asm,
         msg=f"[torch vs  aiter_asm][   Quant]: {us_aiter_asm:>8.2f} us......",
     )
-    return {
-        "No Quant": us_asm_noquant,
-        "Quant": us_aiter_asm,
-        "err_noquant": err_noquant,
-        "err quant": err,
-    }
+    ret["us_asm_fp8"] = us_aiter_asm
+    ret["err fp8"] = err
+
+    q_quant, q_scale = pertoken_quant(query, quant_dtype=aiter.dtypes.fp8)
+    q_scale = q_scale.squeeze(-1)
+
+    out_hip, us_hip = run_aiter_hip(
+        q_quant,
+        k_quant_,
+        asm_V_shuffle(v_quant_),
+        block_tables,
+        seq_lens,
+        ctx_lens,
+        max_qlen,
+        "fp8",
+        num_kv_heads,
+        scale,
+        k_scale_asm,
+        v_scale_asm,
+        q_scale,
+        output_dtype=dtype,
+    )
+    err = checkAllclose(
+        out_ref,
+        out_hip,
+        msg=f"[torch vs  aiter_hip][   Quant]: {us_hip:>8.2f} us......",
+    )
+    ret["us_hip_fp8"] = us_hip
+    ret["err_hip_fp8"] = err
+
+    return ret
 
 
 head_dim = 128
-block_size = 16
+l_block_size = [16]
 l_dtype = ["bf16"]
-l_num_heads = [(5, 1), (8, 1), (16, 1)][:-1]
+l_num_heads = [(5, 1), (8, 1), (10, 1)]
 l_qlen = [1, 2, 3, 4]
-l_ctx_len = [7, 26, 57, 66, 109, 128, 257, 282, 4097]
+l_ctx_len = [7, 26, 57, 66, 109, 128, 257, 282, 4097, 16384]
 l_batch_size = [128]
 
 parser = argparse.ArgumentParser(
@@ -418,7 +536,6 @@ parser.add_argument(
     "-n",
     "--num_heads",
     type=dtypes.str2tuple,
-    choices=l_num_heads,
     default=None,
     help="""Number of heads.
     e.g. -n 8,1""",
@@ -436,7 +553,6 @@ parser.add_argument(
     "-c",
     "--ctx_len",
     type=int,
-    choices=l_ctx_len,
     default=None,
     help="""Context length.
     e.g. -c 128""",
@@ -445,12 +561,18 @@ parser.add_argument(
     "-b",
     "--batch_size",
     type=int,
-    choices=l_batch_size,
     default=None,
     help="""Batch size.
     e.g. -b 128""",
 )
-
+parser.add_argument(
+    "--block_size",
+    type=int,
+    nargs="*",
+    default=l_block_size,
+    help="""Batch size.
+    e.g. -b 128""",
+)
 args = parser.parse_args()
 if args.dtype is None:
     l_dtype = [dtypes.d_dtypes[key] for key in l_dtype]
@@ -464,23 +586,23 @@ if args.ctx_len is not None:
     l_ctx_len = [args.ctx_len]
 if args.batch_size is not None:
     l_batch_size = [args.batch_size]
+l_block_size = args.block_size
 
 for dtype in l_dtype:
     df = []
-    for num_heads in l_num_heads:
-        for qlen in l_qlen:
-            for ctx_len in l_ctx_len:
-                for batch_size in l_batch_size:
-                    ret = test_pa_mtp(
-                        ctx_len,
-                        batch_size,
-                        num_heads,
-                        head_dim,
-                        block_size,
-                        dtype,
-                        qlen,
-                    )
-                    df.append(ret)
+    for num_heads, qlen, ctx_len, batch_size, block_size in itertools.product(
+        l_num_heads, l_qlen, l_ctx_len, l_batch_size, l_block_size
+    ):
+        ret = test_pa_mtp(
+            ctx_len,
+            batch_size,
+            num_heads,
+            head_dim,
+            block_size,
+            dtype,
+            qlen,
+        )
+        df.append(ret)
     df = pd.DataFrame(df)
     aiter.logger.info(f"summary:\n{df}")
     # df.to_csv("mla_prefill.csv")
